@@ -1,5 +1,5 @@
 from pfunk.client import q
-from pfunk.resources import Function, Role
+from pfunk.resources import Function, Role, Index
 
 
 class AuthFunction(Function):
@@ -41,6 +41,16 @@ class LoginUser(AuthFunction):
         )
 
 
+class LogoutUser(AuthFunction):
+
+    def get_body(self):
+        return q.query(
+            q.lambda_([],
+                      q.logout(True)
+                      )
+        )
+
+
 class UpdatePassword(AuthFunction):
     def get_body(self):
         return q.query(q.lambda_(["input"],
@@ -125,6 +135,12 @@ class Public(Role):
                 "actions": {
                     "call": True
                 }
+            },
+            {
+                "resource": q.function("logout_user"),
+                "actions": {
+                    "call": True
+                }
             }
 
         ]
@@ -188,21 +204,21 @@ class GenericAuthorizationRole(Role):
         ]
         priv_list.extend([
             {
-                "resource": q.function(i.get_name()),
+                "resource": q.index(i().get_name()),
                 "actions": {
                     "read": True
                 }
             }
-            for i in self.collection._indexes
+            for i in self.collection.collection_indexes
         ])
         priv_list.extend([
             {
-                "resource": q.function(i.get_name()),
+                "resource": q.function(i(self.collection).get_name()),
                 "actions": {
                     "call": True
                 }
             }
-            for i in self.collection._functions
+            for i in self.collection.collection_functions
         ])
         return priv_list
 
@@ -258,16 +274,18 @@ class GenericGroupBasedRole(GenericAuthorizationRole):
     name_suffix = 'group_based_crud_role'
 
     def get_lambda(self, resource_type):
+        perm = f'{self.collection.get_collection_name()}-{resource_type}'.lower()
         if resource_type == 'write':
             group_ref = q.select(self.current_group_field,
                                  q.select('data', q.var('old_object')))
             lambda_args = ["old_object", "new_object", "object_ref"]
+
             return q.query(
                 q.lambda_(lambda_args,
                           q.and_(
                               q.equals(
                                   # User ID from index
-                                  q.select(0, q.filter_(lambda i: q.equals(resource_type, i),
+                                  q.select(0, q.filter_(lambda i: q.equals(perm, i),
                                                         q.select(self.permissions_field,
                                                                  q.get(
                                                                      q.match(
@@ -276,7 +294,7 @@ class GenericGroupBasedRole(GenericAuthorizationRole):
                                                                          q.current_identity()
                                                                      )
                                                                  )))),
-                                  resource_type
+                                  perm
                               ),
                               q.equals(
                                   q.select(self.current_group_field, q.select('data', q.var('old_object'))),
@@ -298,7 +316,7 @@ class GenericGroupBasedRole(GenericAuthorizationRole):
             q.lambda_(
                 lambda_args,
                 q.equals(
-                    q.select(0, q.filter_(lambda i: q.equals(resource_type, i),
+                    q.select(0, q.filter_(lambda i: q.equals(perm, i),
                                           q.select(self.permissions_field,
                                                    q.select("data",
                                                             q.get(q.match(
@@ -306,7 +324,7 @@ class GenericGroupBasedRole(GenericAuthorizationRole):
                                                                 group_ref,
                                                                 q.current_identity()
                                                             )))))),
-                    resource_type
+                    perm
                 )
             )
         )
